<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * Special pages provided by WikiDB.
 * The code layout for each IncludableSpecialPage object was originally copied from
 * the NewPages extension by Rob Church, with appropriate modifications.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2010, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 466 $
 */

WikiDB_RegisterRevision('$Rev: 466 $');

//////////////////////////
// Credit information

	$pWikiDB_SpecialPages = array(
		'Undefined Tables' 	=> "lists tables that have data but no definition",
		'Empty Tables' 		=> "lists tables that do not contain any data",
		'Defined Tables' 	=> "lists all defined tables",
	);

	foreach ($pWikiDB_SpecialPages as $PageName => $Description) {
		$wgExtensionCredits['specialpage'][] = array(
			'name' => $PageName,
			'author' => $pWikiDB_Author,
			'description' => "part of WikiDB - " . $Description,
			'url' => $pWikiDB_URL,
		);
	}


//////////////////////////
// WikiDB_SpecialPage
// This class extends the standard IncludableSpecialPage to add a couple of standard
// functions that WikiDB needs.

class WikiDB_SpecialPage extends IncludableSpecialPage {

	var $pItemsPerPage = 50;

	function pRegister($SpecialPageName) {
		SpecialPage::SpecialPage($SpecialPageName, '', true, false, 'default', true);
	}

	function pSetItemsPerPage($ItemsPerPage) {
		if (!$ItemsPerPage) {
			global $wgRequest;
			$ItemsPerPage = $wgRequest->getIntOrNull('limit');
			if (!$ItemsPerPage)
				$ItemsPerPage = 50;
		}
		else
			$ItemsPerPage = intval($ItemsPerPage);

		$this->pItemsPerPage = $ItemsPerPage;
	}

	function pMakeLimitLinks() {
		global $wgUser;

		$Skin = $wgUser->getSkin();
		$Title = Title::makeTitle(NS_SPECIAL, $this->getName());
		$Limits = array(10, 20, 30, 50, 100, 150);

		foreach ($Limits as $Limit) {
			if ($Limit != $this->pItemsPerPage) {
				$Links[] = $Skin->makeKnownLinkObj($Title, $Limit,
												   'limit=' . $Limit);
			}
			else
				$Links[] = (string)$Limit;
		}
		return(wfMsgHtml('wikidb-limitlinks', implode(' | ', $Links)));
	}

} // END class WikiDB_SpecialPage


//////////////////////////
// Undefined Tables

class WikiDB_SP_UndefinedTables extends WikiDB_SpecialPage {

	function WikiDB_SP_UndefinedTables() {
		parent::pRegister('UndefinedTables');
	}

	function execute($ItemsPerPage) {
		global $wgOut;

		$this->pSetItemsPerPage($ItemsPerPage);

		# Don't show the navigation if we're including the page
		if (!$this->mIncluding) {
			$this->setHeaders();
			$wgOut->addWikiText(wfMsg('undefinedtables-header',
								$this->pItemsPerPage));
			$wgOut->addHTML($this->pMakeLimitLinks());
		}

		$DB =& wfGetDB(DB_SLAVE);
		$tblTables = $DB->tableName(pWIKIDB_tblTables);
		$tblRows = $DB->tableName(pWIKIDB_tblRows);
		$Join =  $tblRows . " LEFT JOIN " .  $tblTables
			  . " ON " . $tblRows . ".table_namespace = "
				. $tblTables . ".table_namespace"
			  . " AND " . $tblRows . ".table_title = " . $tblTables . ".table_title";
		$GroupFields = $tblRows . ".table_namespace, " . $tblRows . ".table_title";
		$Fields = "COUNT(" . $tblRows . ".table_title) AS RecordCount, "
				. $GroupFields;
		$res = $DB->query("SELECT " . $Fields . " FROM " . $Join . " WHERE "
			 . $tblTables . ".table_title IS NULL " . " GROUP BY " . $GroupFields
			 . " ORDER BY RecordCount DESC, " . $tblRows . ".table_namespace, "
			 . $tblRows . ".table_title " . "LIMIT 0," . $this->pItemsPerPage);

		$RowCount = $DB->numRows($res);
		if ($RowCount > 0) {
		// Make list
			if (!$this->mIncluding) {
				if ($RowCount == 1)
					$wgOut->addWikiText(wfMsg('undefinedtables-showing'));
				else {
					$wgOut->addWikiText(wfMsg('undefinedtables-showing-plural',
										$RowCount));
				}
			}
			$wgOut->addHTML("<ol>\n");
			while($row = $DB->fetchObject($res))
				$wgOut->addHTML($this->pMakeListItem($row));
			$wgOut->addHTML("</ol>\n");
		}
		else
			$wgOut->addWikiText(wfMsg('undefinedtables-none'));
		$DB->freeResult($res);
	}

	function pMakeListItem($Row) {
		global $wgUser;

		$Title = Title::makeTitleSafe($Row->table_namespace, $Row->table_title);
		if (!is_null($Title)) {
			$skin = $wgUser->getSkin();
			$DefLink = $skin->makeLinkObj($Title);
			$DataLink = $Title->getLocalURL('action=viewdata');
			if ($Row->RecordCount == 1)
				$DataLinkText = wfMsg('undefinedtables-record');
			else {
				$DataLinkText = wfMsg('undefinedtables-record-plural',
									  $Row->RecordCount);
			}
			return("<li>" . $DefLink . ' - <a href="' . $DataLink . '">'
				   . $DataLinkText . '</a></li>' . "\n");
		}
		else {
			return("<!-- Invalid title " . htmlspecialchars($Row->table_title)
				 . " in namespace " . htmlspecialchars($Row->table_namespace)
				 . " -->\n");
		}
	}

} // END class WikiDB_SP_UndefinedTables


//////////////////////////
// Empty Tables

class WikiDB_SP_EmptyTables extends WikiDB_SpecialPage {

	function WikiDB_SP_EmptyTables() {
		parent::pRegister('EmptyTables');
	}

	function execute($ItemsPerPage) {
		global $wgOut;

		$this->pSetItemsPerPage($ItemsPerPage);

		# Don't show the navigation if we're including the page
		if (!$this->mIncluding) {
			$this->setHeaders();
			$wgOut->addWikiText(wfMsg('emptytables-header', $this->pItemsPerPage));
			$wgOut->addHTML($this->pMakeLimitLinks());
		}

		$DB =& wfGetDB(DB_SLAVE);
		$tblTables = $DB->tableName(pWIKIDB_tblTables);
		$tblRows = $DB->tableName(pWIKIDB_tblRows);
		$Join =  $tblTables . " LEFT JOIN " .  $tblRows
			  . " ON " . $tblTables . ".table_namespace = "
				. $tblRows . ".table_namespace"
			  . " AND " . $tblTables . ".table_title = " . $tblRows . ".table_title";
		$GroupFields = $tblTables . ".table_namespace, "
					 . $tblTables . ".table_title";
		$Fields = $GroupFields;
		$res = $DB->query("SELECT " . $Fields . " FROM " . $Join . " WHERE "
			 . $tblRows . ".table_title IS NULL"
			 . " GROUP BY " . $GroupFields
			 . " ORDER BY " . $tblTables . ".table_namespace, "
			 . $tblTables . ".table_title"
			 . " LIMIT 0," . $this->pItemsPerPage);

		$RowCount = $DB->numRows($res);
		if ($RowCount > 0) {
		// Make list
			if (!$this->mIncluding) {
				if ($RowCount == 1)
					$wgOut->addWikiText(wfMsg('emptytables-showing'));
				else {
					$wgOut->addWikiText(wfMsg('emptytables-showing-plural',
										$RowCount));
				}
			}
			$wgOut->addHTML("<ol>\n");
			while($row = $DB->fetchObject($res))
				$wgOut->addHTML($this->pMakeListItem($row));
			$wgOut->addHTML("</ol>\n");
		}
		else
			$wgOut->addWikiText(wfMsg('emptytables-none'));
		$DB->freeResult($res);
	}

	function pMakeListItem($Row) {
		global $wgUser;

		$Title = Title::makeTitleSafe($Row->table_namespace, $Row->table_title);
		if (!is_null($Title)) {
			$skin = $wgUser->getSkin();
			$DefLink = $skin->makeLinkObj($Title);
			return( "<li>" . $DefLink . '</li>' . "\n");
		}
		else {
			return("<!-- Invalid title " . htmlspecialchars($Row->table_title)
				 . " in namespace " . htmlspecialchars($Row->table_namespace)
				 . " -->\n");
		}
	}

} // END class WikiDB_SP_EmptyTables


//////////////////////////
// All Tables

class WikiDB_SP_AllTables extends WikiDB_SpecialPage {

	function WikiDB_SP_AllTables() {
		parent::pRegister('AllTables');
	}

	function execute($ItemsPerPage) {
		global $wgOut;

		$this->pSetItemsPerPage($ItemsPerPage);

		# Don't show the navigation if we're including the page
		if (!$this->mIncluding) {
			$this->setHeaders();
			$wgOut->addWikiText(wfMsg('alltables-header', $this->pItemsPerPage));
			$wgOut->addHTML($this->pMakeLimitLinks());
		}

		$DB =& wfGetDB(DB_SLAVE);
		$tblTables = $DB->tableName(pWIKIDB_tblTables);
		$tblRows = $DB->tableName(pWIKIDB_tblRows);
		$Join =  $tblTables . " LEFT JOIN " .  $tblRows
			  . " ON " . $tblTables . ".table_namespace = "
					. $tblRows . ".table_namespace"
			  . " AND " . $tblTables . ".table_title = " . $tblRows . ".table_title";
		$GroupFields = $tblTables . ".table_namespace, "
					 . $tblTables . ".table_title";
		$Fields = "COUNT(" . $tblRows . ".table_title) AS RecordCount, "
				. $GroupFields;
		$res = $DB->query("SELECT " . $Fields . " FROM " . $Join
			 . " GROUP BY " . $GroupFields
			 . " ORDER BY " . $tblTables . ".table_namespace, "
			 . $tblTables . ".table_title"
			 . " LIMIT 0," . $this->pItemsPerPage);

		$RowCount = $DB->numRows($res);
		if ($RowCount > 0) {
		// Make list
			if (!$this->mIncluding) {
				if ($RowCount == 1)
					$wgOut->addWikiText(wfMsg('alltables-showing'));
				else {
					$wgOut->addWikiText(wfMsg('alltables-showing-plural',
											  $RowCount));
				}
			}
			$wgOut->addHTML("<ol>\n");
			while($row = $DB->fetchObject($res))
				$wgOut->addHTML($this->pMakeListItem($row));
			$wgOut->addHTML("</ol>\n");
		}
		else
			$wgOut->addWikiText(wfMsg('alltables-none'));
		$DB->freeResult($res);
	}

	function pMakeListItem($Row) {
		global $wgUser;

		$Title = Title::makeTitleSafe($Row->table_namespace, $Row->table_title);
		if (!is_null($Title)) {
			$skin = $wgUser->getSkin();
			$DefLink = $skin->makeLinkObj($Title);
			$DataLink = $Title->getLocalURL('action=viewdata');
			if ($Row->RecordCount == 1)
				$DataLinkText = wfMsg('alltables-record');
			else
				$DataLinkText = wfMsg('alltables-record-plural', $Row->RecordCount);

			if ($Row->RecordCount == 0)
				$Class = ' class="new"';
			else
				$Class = '';

			return("<li>" . $DefLink . ' - <a href="' . $DataLink . '"'
				   . $Class . '>' . $DataLinkText . '</a></li>' . "\n");
		}
		else {
			return("<!-- Invalid title " . htmlspecialchars($Row->table_title)
				 . " in namespace " . htmlspecialchars($Row->table_namespace)
				 . " -->\n");
		}
	}

} // END class WikiDB_SP_AllTables
