<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * Common utility functions for WikiDB.
 * These may be useful in other apps, but for
 * portability they are added here (with a WikiDB_ prefix for compatibility).
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2010, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 484 $
 */

WikiDB_RegisterRevision('$Rev: 484 $');

// Converts a string to a number.  Either an int or a float as appropriate.
// Zero is returned for non-numeric strings.
	function WikiDB_StrToNumber($Value) {
		if (is_numeric($Value)) {
			if (strpos($Value, ".") !== false)
				return floatval($Value);
			else
				return intval($Value);
		}
		else
			return 0;
	}

// Formats a float value as a string, with trailing zeros removed and a minimum of
// 1 digit before the decimal point.
	function WikiDB_FormatFloat($Value) {
		$Value = sprintf("%1f", $Value);
		return preg_replace("/(\.\d+)0+$/U", "$1", $Value);
	}

// TODO - this is VERY crude at the moment, and may well fail in certain edge cases
// (or even, certain 'non-edge' cases!)
	function pWikiDB_ExpandVariables($Input, $Data) {
		if (is_array($Data) && count($Data) > 0) {
		// Substitute variables.  Note that we need to replace $ with \$ within
		// $Value, to stop it being interpreted as a PHP variable reference.
			foreach ($Data as $Key => $Value) {
				$Input = preg_replace("/{{{" . preg_quote($Key, "/") . "}}}/",
									  str_replace('$', '\$', $Value),
									  $Input);
			}
		}
		return $Input;
	}

// Wrapper function to simplify text parsing.
// We use a local instance of the parser class, so that we don't get into a
// recursive nightmare (e.g. if a <repeat> tag contains a <repeat> tag).
// All tag hooks are ignored (as they are being dealt with already), however
// parser function hooks are parsed as expected.
	function WikiDB_Parse($Input, &$Parser, $LineStart = true) {
		global $wgParser;
		static $LocalParser;
		
		if (!isset($LocalParser)) {
			$LocalParser = new Parser;
		
		// Later versions of MW (tested on 1.16) automatically add the necessary parser 
		// function hooks to a new Parser instance, however earlier versions (tested
		// on 1.6) don't.  If they are not present, copy them from the global
		// $wgParser object.
			if (count($LocalParser->mFunctionHooks) == 0)
				$LocalParser->mFunctionHooks = $wgParser->mFunctionHooks;
		}

		$Output = $LocalParser->parse($Input, $Parser->mTitle, $Parser->mOptions,
									  $LineStart, true);
		return $Output->getText();
	}

// WikiDB_MakeTitle()
// $Title may either be a title object or a string representing the name of a
// page.  If $Title is a string, but doesn't include a namespace, then
// $DefaultNamespace is used if specified.  If no default is specified then 
// WikiDB::GetDefaultTableNamespace() is used to fill in a sensible default.
	function WikiDB_MakeTitle($Title, $DefaultNamespace = NULL) {
	// If $Title is already a Title object, just return it.
	// TODO: Currently we assume that if it is an object, it is a Title object.
	//		 Is this sensible?
		if (is_object($Title)) {
			return $Title;
		}
	// Otherwise, it is a text string representing a page title.
	// In this case we create a new title object from that string, using the
	// default table namespace if necessary.
		else {
		// If no default namespace is specified, use the standard default.
			if ($DefaultNamespace === NULL)
				$DefaultNamespace = WikiDB::GetDefaultTableNamespace();

		// TODO: Should this use Title::newFromDBKey() instead?  If $Title contains
		//		 text it *should* always be a DB key, but it seems that the
		//		 transformations involved in newFromText() won't affect strings that
		//		 are already transformed, therefore it is safe (if marginally slower)
		//		 to pass it through there anyway - and that avoids problems if we
		//		 get passed in the actual title, rather than the DB key, for any
		//		 reason.
			return Title::newFromText($Title, $DefaultNamespace);
		}
	}

if (!function_exists("IsBlank")) {
	function IsBlank($Input) {
	// is_null() returns true if $Input is unset or null
		if (is_null($Input))
			return true;
		elseif ($Input === "")
			return true;
		elseif (is_array($Input) && count($Input) == 0)
			return true; 
		else
			return false;
	}
}


// Returns the first key in an array, without affecting the array pointer.
function GetFirstKey($arr) {
	return GetNumberedKey($arr, 0);
}

function GetNumberedKey($arr, $Column) {
	if (!is_array($arr))
		return NULL;

	$CurrentElement = key($arr);

	reset($arr);
	for ($i = 1; $i < $Column; $i++)
		next($arr);		
	$ReturnVal = key($arr);

	reset($arr);
	while ($CurrentElement != key($arr))
		next($arr);
	return $ReturnVal;

}

function QuoteIdentifier($Identifier) {
	$DB =& wfGetDB(DB_SLAVE);

// For some reason the default DB handler, i.e. MySQL, doesn't have this function.
// Some others do, at least on later version of MW.
// If it exists, call it.
	if (is_callable(array($DB, "ident_quote")))
		return $DB->ident_quote($Identifier);

// Otherwise quote with back-tick.
	return "`" . str_replace("`", "``", $Identifier) . "`";
}