<?php
/**
 * Header file for WikiDB command-line scripts.
 * Used to include necessary files and setup any common variables, etc.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2010, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 479 $
 */

////////////////////////////////////////
// INITIALISE MEDIAWIKI CLI
////////////////////////////////////////

// Constant holding the path to the CLI entry-point, relative to the
// MediaWiki root folder.
	define("MWEntryPoint", "/maintenance/commandLine.inc");

// Check the MW_INSTALL_PATH environment variable, to see if it is set.  If so,
// this is the root folder of the MediaWiki installation.
	$MWPath = getenv("MW_INSTALL_PATH");

// If it is not set, look in the standard location relative to this extension file,
// assuming that the extension was installed in the expected place.
	if ($MWPath === false) {
		$MWPath = dirname(__FILE__) . "/../../..";
	
	// If the entry point can't be found, die with a warning message and suggest
	// the user sets MW_INSTALL_PATH.
		if (!file_exists($MWPath . MWEntryPoint)) {
			die("Unable to locate MediaWiki installation.  "
				. "Try setting the MW_INSTALL_PATH environment variable.\n");
		}
	}

// If MW_INSTALL_PATH was set, but we can't find the entry point, then die with
// a message indicating that the setting is no good.
	elseif (!file_exists($MWPath . MWEntryPoint))
		die("MediaWiki not found at MW_INSTALL_PATH (" . $MWPath . ").\n");

// If we get here, then MediaWiki was found, so load the entry point.
	require_once($MWPath . MWEntryPoint);

////////////////////////////////////////
// INCLUDE SUPPORT FILES
////////////////////////////////////////

// Currently only one support file, as not many support functions required.
// If this grows, they may be split into several files.
	require_once(dirname( __FILE__ ) . "/install_functions.inc");

////////////////////////////////////////
// CREATE STANDARD VARIABLES
////////////////////////////////////////

// Create DB connection to the master DB (for schema updates, etc.)
	wfWaitForSlaves(false);
	$DB =& wfGetDB(DB_MASTER);
